<?php

//module/Secure/src/Secure/Model/UserResourceTable.php

/**************
	Purpose: Model functions for accessing the calendar table
	Author: Kevin Gyberg
	Date: 03/07/2012
*************/

namespace Secure\Model;
use Zend\Db\TableGateway\TableGateway;
use Secure\Model\Events;
use Zend\Db\Sql\Sql, Zend\Db\Sql\Insert, Zend\Db\Sql\Where, Zend\Db\Sql\Predicate;
use Zend\Db\Sql\Select;
use Zend\Db\Adapter\Adapter;

class UserResourceTable
{
	/**
     * @var tableGateway
     */
    protected $tableGateway;
    
    /**
     * @var tableGateway
     */
    protected $resourceObj;
    
    /**
     * @var dbSql
     */
	protected $dbSql;

	/**
     * Constructor to initialize table gateway.
     * @param  TableGateway $tableGateway 
     */
    public function __construct(TableGateway $tableGateway)
    {
        $this->tableGateway = $tableGateway;
		$this->dbSql = $this->tableGateway->getAdapter();
    }
    
    /**
     * Function to save a permission record in the database.
     * 
     * @param array $userResourceInserData
     * 
     */
    public function savePermission($userResourceInsertData, $userId)
    {   
        if ($userResourceInsertData) {
            
            $permissionData = array(
                'fk_userid'         => $userResourceInsertData['userId'],
                'fk_resourceid'     => $userResourceInsertData['resourceId'],
                'perm_type'         => $userResourceInsertData['permType'],
                'userid_created'    => $userId,
                'created'           => date('Y-m-d H:i:s')
            );
            
            if ($this->tableGateway->insert($permissionData)) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Function to update permission in the user_resource table.
     * 
     * @param type $userPermissionInsertData
     * @param type $userId
     * @return boolean
     */
    public function updatePermission($userPermissionInsertData, $userId)
    {
        
        if ($userPermissionInsertData) {
            
            $permissionData = array(
                'perm_type'         => $userPermissionInsertData['permType'],
                'userid_updated' 	=> $userId
            );
            
            if ($this->tableGateway->update(
                            $permissionData,
                            array(
                                'fk_userid'     => $userPermissionInsertData['userId'],
                                'fk_resourceid' => $userPermissionInsertData['resourceId'],
                                'rec_status'    => 1
                            )
                    )) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Check if the permission pre-exist in the database.
     * 
     * @param int $userId
     * @param int $resourceId
     * @param string $permType
     */
    public function isPermissionExist($userId, $resourceId, $permType)
    {
        $rowset = $this->tableGateway->select(
                                            array(
                                                'fk_userid'     => $userId,
                                                'fk_resourceid' => $resourceId,
                                                'perm_type'     => $permType
                                            )
        );
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
     * Check if the permission pre-exist in the database.
     * 
     * @param int $userId
     * @param int $resourceId
     * @param string $permType
     */
    public function isAnyPermissionExist($userId, $resourceId) {
        
        $rowset = $this->tableGateway->select(
                                            array(
                                                'fk_userid'     => $userId,
                                                'fk_resourceid' => $resourceId,
                                                'rec_status'    => 1
                                            )
        );
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
    }
    
    /**
      * Model function to check if a given resource is shared or not.
      * 
      * @param type $userId
      * @param type $resourceId
      */
     public function isResourceShared($userId, $resourceId) {
         
         $rowset = $this->tableGateway->select(
                                            array(
                                                'fk_userid'     => $userId,
                                                'fk_resourceid' => $resourceId,
                                                'rec_status'    => 1
                                            )
        );
         
        $row = $rowset->current();
        if ($row) {
            return $row;
        }
        return false;
     }
    
    /**
     * Get all the records of people with whom this resource is being shared.
     * 
     * @param int $resourceId
     * 
     */
    public function getAllShares($resourceId)
    {
         if ($resourceId) {
            $sql = $this->dbSql;
            $resultSet = $sql->query("SELECT t1.user_resourceid, t1.fk_userid,
                    t1.created, t1.fk_resourceid,
                    t2.email_login, t1.perm_type
                    FROM user_resource AS t1
                    LEFT JOIN users AS t2
                    ON ( t1.fk_userid = t2.userid)
                    WHERE t1.fk_resourceid = $resourceId AND t1.rec_status = 1",
                    Adapter::QUERY_MODE_EXECUTE);
            
            if ($resultSet) {
                return $resultSet;
            }
        }
        return false;
    }
    
    /**
     * Function to delete a user resource record.
     * 
     * @param int $userResourceId
     * @return boolean false
     * 
     */
    public function deleteUserPermission($userResourceId) {
        
        if ($userResourceId) {
            if ($this
                    ->tableGateway
                    ->delete(
                            array(
                                'user_resourceid' => $userResourceId
                            ))
            ) {
                 return true;
            }
        }
        return false;
    }
    
    /**
     * Function to delete a user resource record.
     * 
     * @param int $userResourceId
     * @return boolean false
     * 
     */
    public function deleteUserPermissionByUserId($userPermissionInsertData) {
        
        if ($userPermissionInsertData) {
            
            if ($this->tableGateway->delete(
                        array(
                            'fk_userid'         => $userPermissionInsertData['userId'],
                            'fk_resourceid'     => $userPermissionInsertData['resourceId']
                        )
                    )) {
                return true;
            }
        }
        return false;
    }
    
    /**
     * Recursively make a lookup and determine the permission to be assigned to 
     * a resource.
     * 
     * @param int $resourceId
     * @param int $userId
     * @return boolean return permission type on success and false on failure.
     * 
     */
    public function getPermissionForResource($userId, $resourceId, $parentResourceId)
    {
        if ($resourceId && $userId) {
            
            // check if the given user id is owner of the given resource.
            if ($this->isOwner($userId, $resourceId)) {
                return 'edit';
            }
            
            $rowset = $this->tableGateway->select(
                                                array(
                                                    'fk_userid'     => $userId,
                                                    'fk_resourceid' => $resourceId,
                                                    'rec_status'    => 1
                                                )
            );
            $row = $rowset->current();
            if ($row) {
                return $row['perm_type'];
            }
            else {
                $rowset = $this->tableGateway->select(
                    array(
                        'fk_userid'     => $userId,
                        'fk_resourceid' => $parentResourceId,
                        'rec_status'    => 1
                    )
                );
                $row = $rowset->current();
                
                if ($row) {
                    return $row['perm_type'];
                }
                else {
                    // get the parent resource id and find if there exist a permission of it.
                    $resourceRecord = $this->getResourceTypeById($parentResourceId);
                    
                    if ($resourceRecord) {
                        $rowset = $this->tableGateway->select(
                            array(
                                'fk_userid'     => $userId,
                                'fk_resourceid' => $resourceRecord['fk_resourceid_parent'],
                                'rec_status'    => 1
                            )
                        );
                        $row = $rowset->current();
                        if ($row) {
                            return $row['perm_type'];
                        }
                        else {
                            return 'edit';
                        }
                    }
                }
            }
        }
        return false;
    }
    
    /**
     * Function to check if the given userId is owner of the given resourceId
     * 
     * @param type $userId
     * @param type $resourceId
     */
    public function isOwner($userId, $resourceId) {
        
        if ($userId && $resourceId) {
            $resourceInstance = new TableGateway('resource', $this->dbSql);
            $rowset = $resourceInstance->select(
											array(
												'resourceid'  => $resourceId
											)
            );

            $row = $rowset->current();
            if ($row['fk_userid_owner'] == $userId) {
                return true;
            }
        }
        return false;
    }


    /**
      * Get resource type by id.
      * 
      * @param type $resourceid
      * @return array if record exist and false if not.
      * 
      */
     public function getResourceTypeById($resourceid) {
         
        $resourceInstance = new TableGateway('resource', $this->dbSql);

        $rowset = $resourceInstance->select(
                                       array(
                                           'resourceid'  => $resourceid
                                       )
        );

        $row = $rowset->current();
        if ($row) {
         return $row;
        }
        return false;
     }
     
     /**
      * Check if the permission already exist in the system.
      * 
      * @param string $emailAddress
      * @param int    $userId
      * @param int    $resourceId
      */
     public function isPermissionValid($emailAddress, $userId, $resourceId)
     {
         $rowset = '';
         // if user id exist then the user pre exist in them system.
         if ($userId) {
            // search in the user_resource table.
            $rowset = $this->tableGateway->select(
                                        array(
                                            'fk_userid'     => $userId,
                                            'fk_resourceid' => $resourceId,
                                            'rec_status'    => 1
                                        )
            );
         }
         else {
             // if user id is equal to NULL then the email id/user does not pre-exist
             // in the system.
             
             // search in user_perm_temp table by using email id.
             
             $usersPermTempInstance = new TableGateway('users_perm_temp', $this->dbSql);
             $rowset = $usersPermTempInstance->select(
                                        array(
                                            'fk_email_login'     => $emailAddress,
                                            'fk_resourceid'      => $resourceId,
                                            'rec_status'         => 1
                                        )
             );
        }
         
        $row = $rowset->current();
        if ($row) {
           return $row;
        }
        return false;
     }
     
     /**
      * Function to get the user_resource record.
      * 
      * @param type $userResourceid
      * @return object user_resource record or bool false if failure.
      */
     public function getUserResourceRecord($userResourceid)
     {
         
        if ($userResourceid) {
             
            $rowset = $this->tableGateway->select(
                                           array(
                                               'user_resourceid'    => $userResourceid,
                                               'rec_status'         => 1
                                           )
            );
            
            $row = $rowset->current();
            
            if ($row) {
                
               return $row;
            }
        }

        return false;
     }
}